@@ -2,7 +2,7 @@ package HTTP::MobileAgent::EZweb;
 
 use strict;
 use vars qw($VERSION);
-$VERSION = 0.18;
+$VERSION = 0.20;
 
 use base qw(HTTP::MobileAgent);
 
@@ -24,6 +24,13 @@ sub is_tuka {
   } else {
       return 1 if $tuka eq 'T';
   }
+  return;
+}
+
+sub is_win {
+  my $self = shift;
+  my $win = substr($self->device_id, 2, 1);
+  $win eq '3' ? 1 : 0;
 }
 
 sub parse {
@@ -34,7 +41,6 @@ sub parse {
 	$self->{xhtml_compliant} = 1;
 	my($device, $browser, $opt, $server) = split / /, $ua, 4;
 	$self->{device_id} = $device;
-	$self->{model} = $device;
 
 	my($name, $version) = split m!/!, $browser;
 	$self->{name} = $name;
@@ -53,6 +59,7 @@ sub parse {
 	    $self->{comment} = $comment;
 	}
     }
+    $self->{model} = $self->{device_id};
 }
 
 sub _make_display {
@@ -135,6 +142,18 @@ returns comment like "Google WAP Proxy/1.0". returns undef if nothinng.
 
 returns if the agent is XHTML compliant.
 
+=item is_tuka
+
+  if ($agent->is_tuka) { }
+
+returns if the agent is TU-KA model.
+
+=item is_win
+
+  if ($agent->is_win) { }
+
+returns if the agent is win model.
+
 =back
 
 =head1 TODO
